/* GtkZephyr - a GTK+ based front end to MIT's Zephyr Notification Service
 * Copyright (C) 2000-2001 Sean Dilda
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <zephyr/zephyr.h>
#include <gtk/gtk.h>
#include <string.h>
#include "gtkzephyr.h"
#include "adduser.h"

static GtkWidget *entry;
static GtkWidget *window;


void close_add_window(GtkObject *window, gpointer foo) {

  gtk_editable_delete_text(GTK_EDITABLE(entry), 0, -1);
  gtk_widget_hide(GTK_WIDGET(window));

}



gboolean delete_event_callback(GtkObject *window, GdkEvent *event, gpointer foo) {

  close_add_window(window, NULL);
  return TRUE;

}



void add_user(GtkObject *object) {

  gchar *userid = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
  gchar *userrealm;

  g_strchug(userid);
  g_strchomp(userid);

  if (userid[0] == '\0')
    return;

  if (strchr(userid, '@'))
    userrealm = userid;
  else
    userrealm = g_strdup_printf("%s@%s", userid, ZGetRealm());
  
  if (!gtkzephyr_get_user(userrealm))
    gtkzephyr_add_user(userid);

  close_add_window(GTK_OBJECT(window), NULL);

  if (userid != userrealm)
    g_free(userrealm);
  g_free(userid);

}



void add_user_activate(GtkEditable *entry, gpointer data) {

  add_user(NULL);

}



void gtkzephyr_add_user_dialog(GtkWidget *widget, gpointer data) {
  GtkWidget *vbox, *hbox, *button;

  if (window) {
    gtk_widget_show(window);
    gdk_window_raise(window->window);
  } else {
    window = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_signal_connect_object(GTK_OBJECT(window), "delete_event",
      	GTK_SIGNAL_FUNC(close_add_window), GTK_OBJECT(window));
    gtk_window_set_title(GTK_WINDOW(window), "Add User");
    gtk_container_set_border_width(GTK_CONTAINER(window), 5);
    vbox = gtk_vbox_new(FALSE, 5);
    gtk_container_add(GTK_CONTAINER(window), vbox);
    gtk_widget_show(vbox);
    entry = gtk_entry_new();
    gtk_signal_connect(GTK_OBJECT(entry), "activate",
        GTK_SIGNAL_FUNC(add_user_activate), NULL);
    gtk_editable_set_editable(GTK_EDITABLE(entry), TRUE);
    /* gtk_entry_set_max_length(GTK_ENTRY(entry), 8); */
    gtk_widget_show(entry);
    gtk_box_pack_start(GTK_BOX(vbox), entry, TRUE, TRUE, 2);
    hbox = gtk_hbox_new(FALSE,3);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 2);
    button = gtk_button_new_with_label("Cancel");
    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
	GTK_SIGNAL_FUNC(close_add_window), GTK_OBJECT(window));
    gtk_widget_show(button);
    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 5);
    button = gtk_button_new_with_label("Add");
    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
	GTK_SIGNAL_FUNC(add_user), GTK_OBJECT(window));
    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 5);
    gtk_widget_show(GTK_WIDGET(button));
    gtk_widget_show(GTK_WIDGET(window));
  }
  gtk_widget_grab_focus(entry);

}

