/* GtkZephyr - a GTK+ based front end to MIT's Zephyr Notification Service
 * Copyright (C) 2000 Sean Dilda
 * Copyright (C) 2001 Jack Neely
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <zephyr/zephyr.h>
#include <stdio.h>
#include <stdlib.h>
#include "gtkzephyr.h"
#include "zcalls.h"

/* Helper functions */
GSList *readfile(gchar *filename);
/*
 * Reads a file into a linked list where each line becomes a string,
 *    one string per node.  The first node contains the first string
 *    of the file.
 * filename - pointer to string containing the filename to read
 * Returns a GDList.  On error this will be NULL.
 */

void strip_comments(gchar *str);
/*
 * str is edited to remove all comments denoted by a '#' and all
 *    leading and trailing white space is removed.
 */

/* 
 * Replaced by following function
void gtkzephyr_process_user_file() {

  register char *cp;
  FILE *fp;
  gchar *filename, userid[BUFSIZ];

  filename = g_strconcat(getenv("HOME"), "/.anyone", NULL);
  fp = fopen(filename, "r");
  if (fp) {

    for (;;) {
      if (!fgets(userid,sizeof(userid) + 1,fp))
	break;
      if (userid[0] == '\0' || userid[0] == '\n' || userid[0] == '#')
	continue;
      if ((cp = strchr(userid, '#')))
	*cp = '\0'; allow for comments in the middle of a line 
      cp = userid + strlen(userid) - 1;
      *cp = '\0';
      while(*--cp == ' ')
	*cp = '\0';
      gtkzephyr_make_user(userid);
    }
    fclose(fp);

  } else {
    They have no .anyone file, don't worry about it 
  }
  g_free(filename);

} */


void gtkzephyr_process_user_file()
{
	FILE *fd;
	gchar buff[BUFSIZ], *filename;

	filename = g_strconcat(getenv("HOME"), "/.anyone", NULL);
	fd = fopen(filename, "r");
	if (fd) {
		while ((fd) && (fgets(buff, BUFSIZ, fd))) {
			strip_comments(buff);
			if (buff[0] != '\0')
				gtkzephyr_make_user(buff);
		}
		fclose(fd);
	}
}

/* Old code replaced by that below 
void gtkzephyr_process_subscrip_file() {

  FILE *fp;
  gchar line[BUFSIZ], class_buffer[BUFSIZ], instance[BUFSIZ], recipient[BUFSIZ];
  gchar *class, *temp, *filename;
  gchar c;

  filename = g_strconcat(getenv("HOME"), "/.zephyr.subs", NULL);
  fp = fopen(filename, "r");
  if (fp) {

    for (;;) {
      class = class_buffer;
      if (!fgets(line,sizeof(line) + 1,fp)) {
	break;
      }
      if ((temp = strchr(line, '#')))
	*temp = '\0';
      temp = line + strlen(line) - 1;
      *temp = '\0';
      while (*--temp == ' ' && temp > line)
	*temp = '\0';
      for (temp=line; *temp && *temp==' '; temp++);
      if (*temp == '\0' || *temp == '\n' || *temp == '#') {
	continue;
      }

      sscanf(temp,"%[^,],%[^,],%s", class, instance, recipient);

      if (!g_strcasecmp(DEFAULT_CLASS, class) && !g_strcasecmp(DEFAULT_INSTANCE, instance) && !strcmp(TOKEN_ME, recipient))
	continue;

      c = class[0];
      if (c == UNSUBSCRIBE_CHARACTER || c == PUNT_CHARACTER)
	class++;

      gtkzephyr_macro_sub(class);
      gtkzephyr_macro_sub(instance);
      gtkzephyr_macro_sub(recipient);

      switch (c) {
	case UNSUBSCRIBE_CHARACTER:
	  gtkzephyr_unsub(class, instance, recipient);
  	  break;
	case PUNT_CHARACTER:
	  gtkzephyr_punt(class, instance, recipient);
	  break;
	default:
    	  gtkzephyr_make_inst(class, instance, recipient);
	  break;
      }
    }
    fclose(fp);
  } else {
    Oh darn, no .zephyr.subs file, ignore it 
  }
  g_free(filename);

}    */

void gtkzephyr_process_subscrip_file() {

	FILE *fp;
	gchar line[BUFSIZ], instance[BUFSIZ], recipient[BUFSIZ];
	gchar *filename, *class, *class_buf;
	gchar c;

	filename = g_strconcat(getenv("HOME"), "/.zephyr.subs", NULL);
	fp = fopen(filename, "r");
	if (!fp) {
		DEBUG_INFO("No %s.\n", filename);
		g_free(filename);
	       	return;
	}
	/* We keep class_buf poitning to what g_malloc() returns because
	 * we might end up changing the value of class */
	class_buf = (gchar *)g_malloc(BUFSIZ);
	while (fgets(line, BUFSIZ, fp)) {
		/* reset class.  If we don't and we modified it last time
		 * we might over-run the buffer this time */
		class = class_buf;
		strip_comments(line);
		if (line[0])
			sscanf(line, "%[^,],%[^,],%s", class, 
					instance, recipient);
		if (!g_strcasecmp(DEFAULT_CLASS, class) && 
			!g_strcasecmp(DEFAULT_INSTANCE, instance) && 
			!strcmp(TOKEN_ME, recipient))
			continue;

		c = class[0];
		if ((c == UNSUBSCRIBE_CHARACTER) || (c == PUNT_CHARACTER))
			class++;

		gtkzephyr_macro_sub(class);
		gtkzephyr_macro_sub(instance);
		gtkzephyr_macro_sub(recipient);

		switch (c) {
		case UNSUBSCRIBE_CHARACTER:
			gtkzephyr_unsub(class, instance, recipient);
			break;
		case PUNT_CHARACTER:
			gtkzephyr_punt(class, instance, recipient);
			break;
		default:
			gtkzephyr_make_inst(class, instance, recipient);
			break;
      		}
	}
	fclose(fp);
	g_free(filename);
	g_free(class_buf);
}


/*
 * Original code
 * void gtkzephyr_file_add_user(gchar *userid) {

  FILE *fp;
  gchar *filename;

  DEBUG_INFO("Adding new user to file..\n");

  filename = g_strconcat(getenv("HOME"), "/.anyone", NULL);
  DEBUG_INFO("Filename: %s\n", filename);
  fp = fopen(filename, "a");
  fputs(userid, fp);
  fputs("\n", fp);
  fclose(fp);
  g_free(filename);

}
 */

/* The above function has been edited into the following
 * function by Jack Neely <jjneely@eos.ncsu.edu>
 */
void gtkzephyr_file_add_user(gchar *userid) {

  FILE *fp;
  gchar *filename;
  gchar buff[BUFSIZ];
  int isPresent = 0;

  filename = g_strconcat(getenv("HOME"), "/.anyone", NULL);
  DEBUG_INFO("Reading in .anyone file...\n");
  fp = fopen(filename, "r");
  while (fgets(buff, BUFSIZ, fp)) {
	  strip_comments(buff);
	  if (strncmp(buff, userid, BUFSIZ) == 0) {
		  isPresent = 1;
		  break;
	  }
  }

  if (isPresent) {
	DEBUG_INFO("User %s is allready in %s.\n", userid, filename);
  }
  else {
  	DEBUG_INFO("Adding new user to file..\n");
  	fp = fopen(filename, "a");
  	fputs(userid, fp);
  	fputs("\n", fp);
  	fclose(fp);
  }

  g_free(filename);
}

void gtkzephyr_file_remove_user(gchar *userid)
{
	GSList *users = NULL;
	GSList *ref;
	gchar *tmp, *filename;
	FILE *fd;

	filename = g_strconcat(getenv("HOME"), "/.anyone", NULL);
	DEBUG_INFO("Reading in .anyone file...\n");
	users = readfile(filename);

    	DEBUG_INFO("Writing new %s file...\n", filename);
	fd = fopen(filename, "w");
	ref = users;
	while (ref) {
		tmp = g_strdup(ref->data);
		strip_comments(tmp);
		if (strncmp(tmp, userid, BUFSIZ) != 0)
			fputs(ref->data, fd);
		g_free(tmp);
		ref = g_slist_next(ref);
	}
	fclose(fd);
	g_slist_foreach(users, (GFunc)g_free, NULL);
	g_slist_free(users);
}



void gtkzephyr_file_add_subscrip(gchar *class, gchar *instance, gchar *recipient) 
{
	gchar *filename;
	gchar buff[BUFSIZ], *str;
	FILE *fp;
	gboolean isPresent = FALSE;
	GSList *contents = NULL, *ref;
	/* If there is a punt character or unsubscribe char then
	 * remove that character.  Else check for previous existance. 
	 * If not present then add new subscription.            */
	
	str = g_strjoin(",", class, instance, recipient, NULL);
	DEBUG_INFO("gtkzephyr_file_add_subscrip() created %s\n", str);
	filename = g_strconcat(getenv("HOME"), "/.zephyr.subs", NULL);
	contents = readfile(filename);
	fp = fopen(filename, "w");
	
	ref = contents;
	while (ref) {
		strncpy(buff, ref->data, BUFSIZ);
		strip_comments(buff);
		if (strncmp(buff, str, BUFSIZ) == 0) 
			isPresent = TRUE;
		/* Would be nice if we could also see if the instance is just
		 * commented out */
		if ((buff[0] == '-' || buff[0] == '!') && (strncmp(&(buff[1]), str, BUFSIZ-1) == 0)) {
			isPresent = TRUE;
			fputs((ref->data + 1), fp);
		}
		else
			fputs(ref->data, fp);
		ref = g_slist_next(ref);
	}
	if (!isPresent) {
		fputs(str, fp);
		fputs("\n", fp);
	}
	fclose(fp);
	
	g_free(filename);
	g_free(str);
	g_slist_foreach(contents, (GFunc)g_free, NULL);
	g_slist_free(contents);
}



void gtkzephyr_file_remove_subscrip(gchar *class, gchar *instance, gchar *recipient) {
	gchar *filename, *str;
	gchar buff[BUFSIZ];
	GSList *subs = NULL, *ref;
	FILE *fp;

	DEBUG_INFO("In gtkzephyr_file_remove_subscrip()\n");
	filename = g_strconcat(getenv("HOME"), "/.zephyr.subs", NULL);
	subs = readfile(filename);
	fp = fopen(filename, "w");
	str = g_strjoin(",", class, instance, recipient, NULL);
	
	ref = subs;
	while (ref) {
		strncpy(buff, ref->data, BUFSIZ);
		strip_comments(buff);
		if (strncmp(buff, str, BUFSIZ) == 0) {
			DEBUG_INFO("Found %s and removing.\n", str);
		}
		else
			fputs(ref->data, fp);
		ref = g_slist_next(ref);
	}
	fclose(fp);

	g_free(filename);
	g_free(str);
	g_slist_foreach(subs, (GFunc)g_free, NULL);
	g_slist_free(subs);
}

GSList *readfile(gchar *filename)
{
	GSList *list = NULL;
	FILE *fd;
	gchar buff[BUFSIZ];

	fd = fopen(filename, "r");
	if (!fd) return NULL;

	while (fgets(buff, BUFSIZ, fd))
		list = g_slist_append(list, g_strndup(buff, BUFSIZ));

	fclose(fd);
	return list;
}

void strip_comments(gchar *str)
{
	gchar *tmp;

	if ((tmp = strchr(str, '#')))
		*tmp = '\0';
	g_strchug(str);
	g_strchomp(str);
}


