/* GtkZephyr - a GTK+ based front end to MIT's Zephyr Notification Service
 * Copyright (C) 2000-2001 Sean Dilda
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <zephyr/zephyr.h>
#include <gtk/gtk.h>
#include "gtkzephyr.h"
#include "zcalls.h"
#include "fileio.h"


GtkWidget *window;
GtkWidget *class_entry, *instance_entry, *recipient_entry;
GtkWidget *subscribe_button;



void close_subscribe_window(GtkObject *object, gpointer foo) {

  gtk_editable_delete_text(GTK_EDITABLE(class_entry), 0, -1);
  gtk_editable_delete_text(GTK_EDITABLE(instance_entry), 0, -1);
  gtk_entry_set_text(GTK_ENTRY(recipient_entry), "*");
  gtk_widget_hide(GTK_WIDGET(window));

}



void check_if_can_sub(GtkWidget *widget, gpointer foo) {


  if (GTK_ENTRY(class_entry)->text_length && GTK_ENTRY(instance_entry)->text_length && GTK_ENTRY(recipient_entry)->text_length) {
    if (!GTK_WIDGET_SENSITIVE(subscribe_button))
      gtk_widget_set_sensitive(subscribe_button, TRUE);
  } else if (GTK_WIDGET_SENSITIVE(subscribe_button)) {
    gtk_widget_set_sensitive(subscribe_button, FALSE);
  }

}



void subscribe_inst(GtkObject *object) {

  gtkzephyr_inst *inst =
    gtkzephyr_make_inst(gtk_entry_get_text(GTK_ENTRY(class_entry)),
        gtk_entry_get_text(GTK_ENTRY(instance_entry)),
        gtk_entry_get_text(GTK_ENTRY(recipient_entry)));
  gtkzephyr_sub_instant(inst->class, inst->instance, inst->recipient);
  gtkzephyr_file_add_subscrip(inst->class, inst->instance, inst->recipient);

  close_subscribe_window(NULL, NULL);

}



void gtkzephyr_subscribe_dialog(GtkWidget *widget, gpointer data) {

  GtkWidget *table, *button, *label, *vbox, *hbuttonbox;

  if (!window) {

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_signal_connect_object(GTK_OBJECT(window), "delete_event",
        GTK_SIGNAL_FUNC(close_subscribe_window), GTK_OBJECT(window));
    gtk_window_set_title(GTK_WINDOW(window), "Subscribe");

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(vbox);
    gtk_container_add(GTK_CONTAINER(window), vbox);

    table = gtk_table_new(3, 2, FALSE);
    gtk_widget_show(table);
    gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 5);
    gtk_container_set_border_width(GTK_CONTAINER(table), 5);
    gtk_table_set_row_spacings(GTK_TABLE(table), 5);
    gtk_table_set_col_spacings(GTK_TABLE(table), 5);

    label = gtk_label_new("class:");
    gtk_widget_show(label);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 3);

    class_entry = gtk_entry_new();
    gtk_signal_connect(GTK_OBJECT(class_entry), "changed", check_if_can_sub, NULL);
    gtk_widget_show(class_entry);
    gtk_table_attach(GTK_TABLE(table), class_entry, 1, 2, 0, 1, 0, 0, 0, 3);

    label = gtk_label_new("instance:");
    gtk_widget_show(label);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 3);

    instance_entry = gtk_entry_new();
    gtk_signal_connect(GTK_OBJECT(instance_entry), "changed", check_if_can_sub, NULL);
    gtk_widget_show(instance_entry);
    gtk_table_attach(GTK_TABLE(table), instance_entry, 1, 2, 1, 2, 0, 0, 0, 3);

    label = gtk_label_new("recipient:");
    gtk_widget_show(label);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 3);

    recipient_entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(recipient_entry), "*");
    gtk_signal_connect(GTK_OBJECT(recipient_entry), "changed", check_if_can_sub, NULL);
    gtk_widget_show(recipient_entry);
    gtk_table_attach(GTK_TABLE(table), recipient_entry, 1, 2, 2, 3, 0, 0, 0, 3);

    hbuttonbox = gtk_hbutton_box_new();
    gtk_widget_show(hbuttonbox);
    gtk_container_set_border_width(GTK_CONTAINER(hbuttonbox), 5);
    gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, TRUE, TRUE, 2);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);

    subscribe_button = gtk_button_new_with_label("Subscribe");
    gtk_signal_connect_object(GTK_OBJECT(subscribe_button), "clicked",
        GTK_SIGNAL_FUNC(subscribe_inst), GTK_OBJECT(subscribe_button));
    gtk_widget_set_sensitive(subscribe_button, FALSE);
    gtk_widget_show(subscribe_button);
    gtk_container_add(GTK_CONTAINER(hbuttonbox), subscribe_button);

    button = gtk_button_new_with_label("Cancel");
    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
        GTK_SIGNAL_FUNC(close_subscribe_window), GTK_OBJECT(window));
    gtk_widget_show(button);
    gtk_container_add(GTK_CONTAINER(hbuttonbox), button);

  }

  gtk_widget_show(window);
  gdk_window_raise(window->window);

}
