/* GtkZephyr - a GTK+ based front end to MIT's Zephyr Notification Service
 * Copyright (C) 2000-2001 Sean Dilda
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <gtk/gtk.h>



typedef struct {
  GtkFunction func;
  gpointer arg;
  GtkWidget *dialog;
} yes_func;



gint do_yes(GtkObject *object, gpointer *arg) {

  yes_func *yes_func_struct = (yes_func *) arg;

  (yes_func_struct->func)(yes_func_struct->arg);

  gtk_widget_destroy(yes_func_struct->dialog);

  g_free(yes_func_struct);

  return 1;

}



void gtkzephyr_yesno(gchar *msg, GtkFunction yesfunction, gpointer yesarg) {

  GtkWidget *dialog, *label, *button, *hbuttonbox;
  yes_func *yes_func_struct;

  yes_func_struct = g_new(yes_func, 1);

  yes_func_struct->func = yesfunction;
  yes_func_struct->arg = yesarg;

  dialog = gtk_dialog_new();
  yes_func_struct->dialog = dialog;

  gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 5);
  
  label = gtk_label_new(msg);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 5);

  hbuttonbox = gtk_hbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_SPREAD);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), hbuttonbox);

  button = gtk_button_new_with_label("Yes");
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(do_yes),
      yes_func_struct);
  gtk_container_add(GTK_CONTAINER(hbuttonbox), button);

  button = gtk_button_new_with_label("No");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
      GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dialog));
  gtk_container_add(GTK_CONTAINER(hbuttonbox), button);
  

  gtk_widget_show_all(dialog);

}
